"""
Some of the code is imported from alphafold3-pytorch.
See https://github.com/lucidrains/alphafold3-pytorch/blob/main/alphafold3_pytorch/common/amino_acid_constants.py.

.. admonition:: License
    :collapsible: closed

    .. code-block::

        MIT License

        Copyright (c) 2024 Phil Wang

        Permission is hereby granted, free of charge, to any person obtaining a copy
        of this software and associated documentation files (the "Software"), to deal
        in the Software without restriction, including without limitation the rights
        to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the Software is
        furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be included in all
        copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
        IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
        SOFTWARE.
"""

CRYSTALLIZATION_AIDS = {
    "SO4",
    "GOL",
    "EDO",
    "PO4",
    "ACT",
    "PEG",
    "DMS",
    "TRS",
    "PGE",
    "PG4",
    "FMT",
    "EPE",
    "MPD",
    "MES",
    "CD",
    "IOD",
}
CRYSTALLOGRAPHY_METHODS = {"X-RAY DIFFRACTION": CRYSTALLIZATION_AIDS}

IONS = {
    "118",
    "119",
    "1AL",
    "1CU",
    "2FK",
    "2HP",
    "2OF",
    "3CO",
    "3MT",
    "3NI",
    "3OF",
    "4MO",
    "4PU",
    "4TI",
    "543",
    "6MO",
    "AG",
    "AL",
    "ALF",
    "AM",
    "ATH",
    "AU",
    "AU3",
    "AUC",
    "BA",
    "BEF",
    "BF4",
    "BO4",
    "BR",
    "BS3",
    "BSY",
    "CA",
    "CAC",
    "CD",
    "CD1",
    "CD3",
    "CD5",
    "CE",
    "CF",
    "CHT",
    "CO",
    "CO5",
    "CON",
    "CR",
    "CS",
    "CSB",
    "CU",
    "CU1",
    "CU2",
    "CU3",
    "CUA",
    "CUZ",
    "CYN",
    "DME",
    "DMI",
    "DSC",
    "DTI",
    "DY",
    "E4N",
    "EDR",
    "EMC",
    "ER3",
    "EU",
    "EU3",
    "F",
    "FE",
    "FE2",
    "FPO",
    "GA",
    "GD3",
    "GEP",
    "HAI",
    "HG",
    "HGC",
    "HO3",
    "IN",
    "IR",
    "IR3",
    "IRI",
    "IUM",
    "K",
    "KO4",
    "LA",
    "LCO",
    "LCP",
    "LI",
    "LU",
    "MAC",
    "MG",
    "MH2",
    "MH3",
    "MMC",
    "MN",
    "MN3",
    "MN5",
    "MN6",
    "MO",
    "MO1",
    "MO2",
    "MO3",
    "MO4",
    "MO5",
    "MO6",
    "MOO",
    "MOS",
    "MOW",
    "MW1",
    "MW2",
    "MW3",
    "NA2",
    "NA5",
    "NA6",
    "NAO",
    "NAW",
    "NET",
    "NI",
    "NI1",
    "NI2",
    "NI3",
    "NO2",
    "NRU",
    "O4M",
    "OAA",
    "OC1",
    "OC2",
    "OC3",
    "OC4",
    "OC5",
    "OC6",
    "OC7",
    "OC8",
    "OCL",
    "OCM",
    "OCN",
    "OCO",
    "OF1",
    "OF2",
    "OF3",
    "OH",
    "OS",
    "OS4",
    "OXL",
    "PB",
    "PBM",
    "PD",
    "PER",
    "PI",
    "PO3",
    "PR",
    "PT",
    "PT4",
    "PTN",
    "RB",
    "RH3",
    "RHD",
    "RU",
    "SB",
    "SE4",
    "SEK",
    "SM",
    "SMO",
    "SO3",
    "T1A",
    "TB",
    "TBA",
    "TCN",
    "TEA",
    "TH",
    "THE",
    "TL",
    "TMA",
    "TRA",
    "V",
    "VN3",
    "VO4",
    "W",
    "WO5",
    "Y1",
    "YB",
    "YB2",
    "YH",
    "YT3",
    "ZCM",
    "ZN",
    "ZN2",
    "ZN3",
    "ZNO",
    "ZO3",
    "ZR",
}


LIGAND_EXCLUSION_LIST = {
    "144",
    "15P",
    "1PE",
    "2F2",
    "2JC",
    "3HR",
    "3SY",
    "7N5",
    "7PE",
    "9JE",
    "AAE",
    "ABA",
    "ACE",
    "ACN",
    "ACT",
    "ACY",
    "AZI",
    "BAM",
    "BCN",
    "BCT",
    "BDN",
    "BEN",
    "BME",
    "BO3",
    "BTB",
    "BTC",
    "BU1",
    "C8E",
    "CAD",
    "CAQ",
    "CBM",
    "CCN",
    "CIT",
    "CL",
    "CLR",
    "CM",
    "CMO",
    "CO3",
    "CPT",
    "CXS",
    "D10",
    "DEP",
    "DIO",
    "DMS",
    "DN",
    "DOD",
    "DOX",
    "EDO",
    "EEE",
    "EGL",
    "EOH",
    "EOX",
    "EPE",
    "ETF",
    "FCY",
    "FJO",
    "FLC",
    "FMT",
    "FW5",
    "GOL",
    "GSH",
    "GTT",
    "GYF",
    "HED",
    "IHP",
    "IHS",
    "IMD",
    "IOD",
    "IPA",
    "IPH",
    "LDA",
    "MB3",
    "MEG",
    "MES",
    "MLA",
    "MLI",
    "MOH",
    "MPD",
    "MRD",
    "MSE",
    "MYR",
    "N",
    "NA",
    "NH2",
    "NH4",
    "NHE",
    "NO3",
    "O4B",
    "OHE",
    "OLA",
    "OLC",
    "OMB",
    "OME",
    "OXA",
    "P6G",
    "PE3",
    "PE4",
    "PEG",
    "PEO",
    "PEP",
    "PG0",
    "PG4",
    "PGE",
    "PGR",
    "PLM",
    "PO4",
    "POL",
    "POP",
    "PVO",
    "SAR",
    "SCN",
    "SEO",
    "SEP",
    "SIN",
    "SO4",
    "SPD",
    "SPM",
    "SR",
    "STE",
    "STO",
    "STU",
    "TAR",
    "TBU",
    "TME",
    "TPO",
    "TRS",
    "UNK",
    "UNL",
    "UNX",
    "UPL",
    "URE",
}


# =====RNA constants=====
rna_restypes = ["A", "C", "G", "U"]
rna_min_restype_num = 20 + 1
rna_restype_order = {
    restype: rna_min_restype_num + i for i, restype in enumerate(rna_restypes)
}
rna_restype_order_with_x = {
    **rna_restype_order,
    "X": len(rna_restypes) + rna_min_restype_num,
}
rna_restype_num = rna_min_restype_num + len(rna_restypes)  # := 21 + 4 := 25.

# Define residue metadata for all unknown RNA residues.
rna_unk_restype = "N"
rna_restype_1to3 = {"A": "A", "C": "C", "G": "G", "U": "U", "X": "N"}
rna_restype_3to1 = {v: k for k, v in rna_restype_1to3.items()}
rna_resnames = [rna_restype_1to3[r] for r in rna_restypes] + [rna_unk_restype]
rna_restype_to_atom24 = {
    "A": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "O2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "N6",
        "N1",
        "C2",
        "N3",
        "C4",
        "",
    ],
    "C": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "O2'",
        "C1'",
        "N1",
        "C2",
        "O2",
        "N3",
        "C4",
        "N4",
        "C5",
        "C6",
        "",
        "",
        "",
    ],
    "G": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "O2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "O6",
        "N1",
        "C2",
        "N2",
        "N3",
        "C4",
    ],
    "U": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "O2'",
        "C1'",
        "N1",
        "C2",
        "O2",
        "N3",
        "C4",
        "O4",
        "C5",
        "C6",
        "",
        "",
        "",
    ],
    "N": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "O2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "N6",
        "N1",
        "C2",
        "N3",
        "C4",
        "",
    ],
}

# =====DNA constants=====

# This is the standard residue order when coding DNA type as a number.
# Reproduce it by taking 3-letter DNA codes and sorting them alphabetically.
dna_restypes = ["A", "C", "G", "T"]
dna_min_restype_num = (
    (20 + 1) + (4 + 1)
)  # := 26. # 20 amino acids + 1 unknown amino acid + 4 RNA residues + 1 unknown RNA residue
dna_restype_order = {
    restype: dna_min_restype_num + i for i, restype in enumerate(dna_restypes)
}
dna_restype_order_with_x = {
    **dna_restype_order,
    "X": len(dna_restypes) + dna_min_restype_num,
}
dna_restype_num = dna_min_restype_num + len(dna_restypes)  # := 26 + 4 := 30.


dna_restype_1to3 = {
    "A": "DA",
    "C": "DC",
    "G": "DG",
    "T": "DT",
    "X": "DN",
}


# NB: restype_3to1 differs from e.g., Bio.Data.PDBData.nucleic_letters_3to1
# by being a simple 1-to-1 mapping of 3 letter names to one letter names.
# The latter contains many more, and less common, three letter names as
# keys and maps many of these to the same one letter name.
dna_restype_3to1 = {v: k for k, v in dna_restype_1to3.items()}

# A compact atom encoding with 24 columns for DNA residues.
# From: https://files.rcsb.org/ligands/view/DA.cif - https://files.rcsb.org/ligands/view/DT.cif
dna_restype_to_atom24 = {
    "DA": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "N6",
        "N1",
        "C2",
        "N3",
        "C4",
        "",
        "",
    ],
    "DC": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "C1'",
        "N1",
        "C2",
        "O2",
        "N3",
        "C4",
        "N4",
        "C5",
        "C6",
        "",
        "",
        "",
        "",
    ],
    "DG": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "O6",
        "N1",
        "C2",
        "N2",
        "N3",
        "C4",
        "",
    ],
    "DT": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "C1'",
        "N1",
        "C2",
        "O2",
        "N3",
        "C4",
        "O4",
        "C5",
        "C7",
        "C6",
        "",
        "",
        "",
    ],
    "DN": [
        "OP3",
        "P",
        "OP1",
        "OP2",
        "O5'",
        "C5'",
        "C4'",
        "O4'",
        "C3'",
        "O3'",
        "C2'",
        "C1'",
        "N9",
        "C8",
        "N7",
        "C5",
        "C6",
        "N6",
        "N1",
        "C2",
        "N3",
        "C4",
        "",
        "",
    ],
}

dna_backbone = [
    "P",
    "OP1",
    "OP2",
    "OP3",
    "C1'",
    "C2'",
    "C3'",
    "C4'",
    "C5'",
    "O3'",
    "O4'",
    "O5'",
]
rna_backbone = [*dna_backbone, "O2'"]
